@@ -17,38 +17,36 @@
 
 #include "test_macros.h"
 
-TEST_CONSTEXPR_CXX14 bool tests()
-{
-    {
-        typedef std::array<int, 2> C;
-        C c = {};
-        ASSERT_NOEXCEPT(c.max_size());
-        assert(c.max_size() == 2);
-    }
-    {
-        typedef std::array<int, 0> C;
-        C c = {};
-        ASSERT_NOEXCEPT(c.max_size());
-        assert(c.max_size() == 0);
-    }
+TEST_CONSTEXPR_CXX14 bool tests() {
+  {
+    typedef std::array<int, 2> C;
+    C c = {};
+    ASSERT_NOEXCEPT(c.max_size());
+    assert(c.max_size() == 2);
+  }
+  {
+    typedef std::array<int, 0> C;
+    C c = {};
+    ASSERT_NOEXCEPT(c.max_size());
+    assert(c.max_size() == 0);
+  }
 
-    return true;
+  return true;
 }
 
-int main(int, char**)
-{
-    tests();
+int main(int, char**) {
+  tests();
 #if TEST_STD_VER >= 14
-    static_assert(tests(), "");
+  static_assert(tests(), "");
 #endif
 
 #if TEST_STD_VER >= 11
-    // Sanity check for constexpr in C++11
-    {
-        constexpr std::array<int, 3> array = {};
-        static_assert(array.max_size() == 3, "");
-    }
+  // Sanity check for constexpr in C++11
+  {
+    constexpr std::array<int, 3> array = {};
+    static_assert(array.max_size() == 3, "");
+  }
 #endif
 
-    return 0;
+  return 0;
 }
